//---------------------------------------------------------------------------
#ifndef MultiMediaTimerHpp
#define MultiMediaTimerHpp

#ifndef WIN32
  #define WIN32
#endif

// C function
#include <stdlib.h>

// C++Bulder classses
#include <Classes.hpp>

// Windows header
#include <Mmsystem.h>

#define ST_MMTIMER_INIT			0
#define ST_MMTIMER_STANDBY		1
#define ST_MMTIMER_EXECUTING	2
#define ST_MMTIMER_STOP			3

#define DEFAULT_MMTIMER_INTERVAL	10
#define MMTIMER_TARGET_RESOLUTION	1	// 1-millisecond target resolution

void CALLBACK gMultiMediaTimerProc(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2);
//---------------------------------------------------------------------------
// Class Definition
//---------------------------------------------------------------------------
class TMultiMediaTimer
{
private:
	int m_nStatus;			// Status
	UINT m_wTimerRes;		// Timer Resolution
	bool m_flgCallback;		// Flag: Callback Executing
	MMRESULT m_hMMTimer;	// Handle of Multimedia Timer
	void __fastcall (__closure *m_pfncMMTimerProc)(void);	// Handle of function to use in callback
	int __fastcall max(int value1, int value2);
	int __fastcall min(int value1, int value2);
public:

	TMultiMediaTimer();		// Constructor
	~TMultiMediaTimer();	// Destructor
	bool __fastcall StartTimer(int nInterval, void __fastcall (__closure *fnc)(void));
	bool __fastcall StopTimer();
	// for internal public
	bool __fastcall IsTimerProcExecuting();
	void __fastcall SetTimerProcExecuting();
	void __fastcall ClearTimerProcExecuting();
	bool __fastcall IsTimerFuncAvailable();
	void __fastcall ExecuteTimerProc();
};

// Function Definition
//---------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////////////////
/**
 \brief    Constructor
*////////////////////////////////////////////////////////////////////////////////////////
TMultiMediaTimer::TMultiMediaTimer()
{
	this->m_nStatus         = ST_MMTIMER_INIT;
	this->m_wTimerRes       = MMTIMER_TARGET_RESOLUTION;
	this->m_flgCallback     = false;
	this->m_hMMTimer        = NULL;
	this->m_pfncMMTimerProc = NULL;
}
/////////////////////////////////////////////////////////////////////////////////////////
/**
 \brief    Destructor
*////////////////////////////////////////////////////////////////////////////////////////
TMultiMediaTimer::~TMultiMediaTimer()
{
	this->StopTimer();
}
/////////////////////////////////////////////////////////////////////////////////////////
/**
 \param [in] nValue1    Value 1.
 \param [in] nValue2    Value 2.

 \return max value.

 \brief    Compare value 1 and value 2
*////////////////////////////////////////////////////////////////////////////////////////
int __fastcall TMultiMediaTimer::max(int nValue1, int nValue2)
{
	return ( (nValue1 > nValue2) ? nValue1 : nValue2 );
}
/////////////////////////////////////////////////////////////////////////////////////////
/**
 \param [in] nValue1    Value 1.
 \param [in] nValue2    Value 2.

 \return min value.

 \brief    Compare value 1 and value 2
*////////////////////////////////////////////////////////////////////////////////////////
int __fastcall TMultiMediaTimer::min(int nValue1, int nValue2)
{
	return ( (nValue1 > nValue2) ? nValue2 : nValue1 );
}
/////////////////////////////////////////////////////////////////////////////////////////
/**
 \param [in] nInterval  Timer interval value.
 \param [in] fnc        Handle of cyclic function.

 \brief    Start Multi Media Tiemr
*////////////////////////////////////////////////////////////////////////////////////////
bool __fastcall TMultiMediaTimer::StartTimer(int nInterval, void __fastcall (__closure *fnc)(void))
{
	TIMECAPS tTimeCaps;	// Timer resolution
	
	if( this->m_nStatus == ST_MMTIMER_INIT )
	{
		if( timeGetDevCaps(&tTimeCaps, sizeof(TIMECAPS)) != TIMERR_NOERROR )
		{
			return false;
		}
		this->m_wTimerRes = min(max(tTimeCaps.wPeriodMin, MMTIMER_TARGET_RESOLUTION), tTimeCaps.wPeriodMax);
		timeBeginPeriod(this->m_wTimerRes);
		this->m_nStatus = ST_MMTIMER_STANDBY;
	}
	if( this->m_nStatus == ST_MMTIMER_STANDBY )
	{
		this->m_pfncMMTimerProc = fnc;
		this->m_hMMTimer = timeSetEvent(nInterval, 
										this->m_wTimerRes,
										gMultiMediaTimerProc,
										(DWORD)this,
										TIME_PERIODIC);
		if( this->m_hMMTimer == NULL )
		{
			this->m_pfncMMTimerProc = NULL;
			this->StopTimer();
			return false;
		}
		this->m_nStatus = ST_MMTIMER_EXECUTING;
	}
	return true;
}
/////////////////////////////////////////////////////////////////////////////////////////
/**

 \brief    Stop Multi Media Tiemr
*////////////////////////////////////////////////////////////////////////////////////////
bool __fastcall TMultiMediaTimer::StopTimer()
{
	if( this->m_nStatus > ST_MMTIMER_STANDBY )
	{
		if( this->m_hMMTimer != NULL )
		{
			timeKillEvent(this->m_hMMTimer);
			this->m_hMMTimer        = NULL;
			this->m_pfncMMTimerProc = NULL;
			this->m_flgCallback     = false;
			this->m_nStatus         = ST_MMTIMER_STANDBY;
		}
	}
	if( this->m_nStatus == ST_MMTIMER_STANDBY )
	{
		timeEndPeriod(this->m_wTimerRes);
		this->m_wTimerRes = MMTIMER_TARGET_RESOLUTION;
	}
	this->m_nStatus = ST_MMTIMER_INIT;
	return true;
}
/////////////////////////////////////////////////////////////////////////////////////////
/**

 \brief    Check Timer Procedure Executing
*////////////////////////////////////////////////////////////////////////////////////////
bool __fastcall TMultiMediaTimer::IsTimerProcExecuting()
{
	return this->m_flgCallback;
}
/////////////////////////////////////////////////////////////////////////////////////////
/**

 \brief    Set Timer Procedure Executing
*////////////////////////////////////////////////////////////////////////////////////////
void __fastcall TMultiMediaTimer::SetTimerProcExecuting()
{
	this->m_flgCallback = true;
}
/////////////////////////////////////////////////////////////////////////////////////////
/**

 \brief    Clear Timer Procedure Executing
*////////////////////////////////////////////////////////////////////////////////////////
void __fastcall TMultiMediaTimer::ClearTimerProcExecuting()
{
	this->m_flgCallback = false;
}
/////////////////////////////////////////////////////////////////////////////////////////
/**

 \brief    Check Timer Procedure Available
*////////////////////////////////////////////////////////////////////////////////////////
bool __fastcall TMultiMediaTimer::IsTimerFuncAvailable()
{
	bool bRslt = (this->m_pfncMMTimerProc == NULL) ? false : true;
	return bRslt;
}
/////////////////////////////////////////////////////////////////////////////////////////
/**

 \brief    Check Timer Procedure Available
*////////////////////////////////////////////////////////////////////////////////////////
void __fastcall TMultiMediaTimer::ExecuteTimerProc()
{
	this->m_pfncMMTimerProc();
}
/////////////////////////////////////////////////////////////////////////////////////////
/**

 \brief    Callback function for Multi Media Tiemr
*////////////////////////////////////////////////////////////////////////////////////////
void CALLBACK gMultiMediaTimerProc(UINT uID, UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
{
	TMultiMediaTimer *ptMMTimer = (TMultiMediaTimer *)dwUser;
	if( ptMMTimer->IsTimerProcExecuting() )
	{
		return;
	}
	ptMMTimer->SetTimerProcExecuting();
	try
	{
		if( ptMMTimer->IsTimerFuncAvailable() )
		{
			ptMMTimer->ExecuteTimerProc();
		}
	}
	__finally
	{
		ptMMTimer->ClearTimerProcExecuting();
	}
}
#endif // MultiMediaTimerHpp
