//---------------------------------------------------------------------------
#ifndef ProjectUtilityHpp
#define ProjectUtilityHpp

// Attention!!
// If you use this class, you should set "NO_WIN32_LEAN_AND_MEAN" to defined section of a compile option.

#include <Forms.hpp>
#include <shlobj.h>
#include <iostream>

/* defines */
#ifndef INDEX_YES_BUTTON
	#define INDEX_YES_BUTTON	2
	#define INDEX_NO_BUTTON		3
	#define INDEX_CANCEL_BUTTON	3
#endif

class TPrjUtil
{
private:
	TForm *m_fmOwner;
	int X;
	int Y;
public:
	TPrjUtil();
	~TPrjUtil();
	WORD wPrimaryLang;
	void __fastcall SetDlgOwner(TForm *fm);
	void __fastcall SetDlgPos(TForm *fm);
	void __fastcall SetDlgPos(int X, int Y);
	int __fastcall ShowMessageDlg(AnsiString Msg, TMsgDlgType DlgType, TMsgDlgButtons Buttons);
	int __fastcall GetSystemFolder(AnsiString &asFolder,int nFolder);
	DWORD __fastcall GetLastErrorText(AnsiString &asErr);
};

// Construct
TPrjUtil::TPrjUtil()
{
	this->m_fmOwner = NULL;
	this->X = 0;
	this->Y = 0;
	wPrimaryLang = PRIMARYLANGID(GetUserDefaultLangID());
}

// Destruct
TPrjUtil::~TPrjUtil()
{
}

// Set the position of dialog window (set to the center of form)
void __fastcall TPrjUtil::SetDlgPos(TForm *fm)
{
	this->Y = fm->Top  + fm->Height/2;
	this->X = fm->Left + fm->Width/2;
}

// Set the position of dialog window ()
void __fastcall TPrjUtil::SetDlgPos(int X, int Y)
{
	this->X = X;
	this->Y = Y;
}

void __fastcall TPrjUtil::SetDlgOwner(TForm *fm)
{
	this->m_fmOwner = fm;
}

// Show Message Dialog
//---------------------------------------------------------------------------
/** Show message dialog at center of form
 *
 *
 * @param [in] Msg is the message
 * @param [in] DlgType is the Dialog Type (TMsgDlgType)
 * @param [in] Buttons is the Button Type (TMsgDlgButtons)
 *
 * @return result
 *	-  = 0 : success
 *	- != 0 : API fault
 */
int __fastcall TPrjUtil::ShowMessageDlg(AnsiString Msg,
					  TMsgDlgType DlgType,
					  TMsgDlgButtons Buttons)
{
	int nRslt;
	AnsiString AS;

	TForm *fmMsg = CreateMessageDialog(Msg,DlgType,Buttons);
	fmMsg->FormStyle = fsStayOnTop;		// View of Top

	//
	// Change Title
	//......................
	switch(DlgType)
	{
	case mtWarning:
		AS = "Warning";
		break;
	case mtError:
		AS = "Error";
		break;
	case mtInformation:
		AS = "Information";
		break;
	case mtConfirmation:
		AS = "Confirmation";
		break;
	case mtCustom:
		AS = "";
		break;
	}
	fmMsg->Caption = AS;
	//
	// Change button caption
	//.........................
	if(Buttons.Contains(mbYes))
		fmMsg->Controls[INDEX_YES_BUTTON]->SetTextBuf(L"Yes");
	if(Buttons.Contains(mbNo))
		fmMsg->Controls[INDEX_NO_BUTTON]->SetTextBuf(L"No");
	if(Buttons.Contains(mbCancel))
		fmMsg->Controls[INDEX_CANCEL_BUTTON]->SetTextBuf(L"Cancel");

	//
	// Set V&H
	//......................
	if( this->m_fmOwner == NULL )
	{
		fmMsg->Top  = this->Y - fmMsg->Height/2;
		fmMsg->Left = this->X - fmMsg->Width/2;
	}
	else
	{
		fmMsg->Top  = this->m_fmOwner->Top + this->m_fmOwner->Height/2 - fmMsg->Height/2;
		fmMsg->Left = this->m_fmOwner->Left + this->m_fmOwner->Width/2 - fmMsg->Width/2;
	}

	//
	// Show
	//......................
	nRslt = fmMsg->ShowModal();

	delete fmMsg;

	return nRslt;

}

//---------------------------------------------------------------------------
/** Get system folder path
 *
 *
 * @param [out] asFolder is the folder
 * @param [in] nFolder is the CSIDL
 *
 * @return result
 *	-  = 0 : success
 *	- != 0 : API fault
 */
int __fastcall TPrjUtil::GetSystemFolder(AnsiString &asFolder,
										 int nFolder)
{
	wchar_t path[MAX_PATH];

	int nRslt = SHGetFolderPath(NULL, nFolder, NULL, NULL, path);
	if( nRslt != 0 )
	{
		asFolder = "";
	}
	else
	{
		asFolder = (AnsiString)path;
	}
	return nRslt;
}

//---------------------------------------------------------------------------
/** Get last error text for Windows
 *
 *
 * @param [out] asErr is the error message
 *
 * @return N/A
 *	- 
 */
DWORD __fastcall TPrjUtil::GetLastErrorText(AnsiString &asErr)
{
	LPVOID  lpMsgBuf;
	DWORD   dwErr;

	dwErr = GetLastError();
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
				  FORMAT_MESSAGE_FROM_SYSTEM |
				  FORMAT_MESSAGE_IGNORE_INSERTS,
				  NULL,
				  dwErr,
				  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // default language
				  (LPTSTR) &lpMsgBuf,
				  0,
				  NULL
				  );
	// set error buffer
	asErr.printf("[ERROR] %s : %s",FormatDateTime("yyyy/mm/dd hh:nn:ss:zzz",Now()).c_str(), (LPCTSTR)lpMsgBuf);
	asErr.Delete(asErr.AnsiPos('\r'),2);

	// release buffer
	LocalFree(lpMsgBuf);
	return dwErr;
}
#endif
